@@ -18,66 +18,113 @@
 #  include <unistd.h>
 #endif
 
+lfDatabase::lfDatabase ()
+{
+    HomeDataDir = g_build_filename (g_get_user_data_dir (),
+                                    CONF_PACKAGE, NULL);
+    UserUpdatesDir = g_build_filename (HomeDataDir, "updates", DATABASE_SUBDIR, NULL);
+    Mounts = g_ptr_array_new ();
+    g_ptr_array_add ((GPtrArray *)Mounts, NULL);
+    Cameras = g_ptr_array_new ();
+    g_ptr_array_add ((GPtrArray *)Cameras, NULL);
+    Lenses = g_ptr_array_new ();
+    g_ptr_array_add ((GPtrArray *)Lenses, NULL);
+}
+
+lfDatabase::~lfDatabase ()
+{
+    size_t i;
+    for (i = 0; i < ((GPtrArray *)Mounts)->len - 1; i++)
+         delete static_cast<lfMount *> (g_ptr_array_index ((GPtrArray *)Mounts, i));
+    g_ptr_array_free ((GPtrArray *)Mounts, TRUE);
+
+    for (i = 0; i < ((GPtrArray *)Cameras)->len - 1; i++)
+         delete static_cast<lfCamera *> (g_ptr_array_index ((GPtrArray *)Cameras, i));
+    g_ptr_array_free ((GPtrArray *)Cameras, TRUE);
+
+    for (i = 0; i < ((GPtrArray *)Lenses)->len - 1; i++)
+         delete static_cast<lfLens *> (g_ptr_array_index ((GPtrArray *)Lenses, i));
+    g_ptr_array_free ((GPtrArray *)Lenses, TRUE);
+
+    g_free (HomeDataDir);
+    g_free (UserUpdatesDir);
+}
+
 lfDatabase *lfDatabase::Create ()
 {
-    return new lfExtDatabase ();
+    return new lfDatabase ();
 }
 
 void lfDatabase::Destroy ()
 {
-    delete static_cast<lfExtDatabase *> (this);
+    delete this;
+}
+
+bool lfDatabase::LoadDirectory (const gchar *dirname)
+{
+    bool database_found = false;
+
+    GDir *dir = g_dir_open (dirname, 0, NULL);
+    if (dir)
+    {
+        GPatternSpec *ps = g_pattern_spec_new ("*.xml");
+        if (ps)
+        {
+            const gchar *fn;
+            while ((fn = g_dir_read_name (dir)))
+            {
+                size_t sl = strlen (fn);
+                if (g_pattern_match (ps, sl, fn, NULL))
+                {
+                    gchar *ffn = g_build_filename (dirname, fn, NULL);
+                    /* Ignore errors */
+                    if (Load (ffn) == LF_NO_ERROR)
+                        database_found = true;
+                    g_free (ffn);
+                }
+            }
+            g_pattern_spec_free (ps);
+        }
+        g_dir_close (dir);
+    }
+
+    return database_found;
 }
 
 lfError lfDatabase::Load ()
 {
-    gchar *dirs [10];
-    const gchar *const *tmp;
-    int ndirs = 0;
+    bool database_found = false;
 
-    dirs [ndirs++] = HomeDataDir;
-#ifdef CONF_DATADIR
-    dirs [ndirs++] = (char *)CONF_DATADIR;
+#ifndef PLATFORM_WINDOWS
+    gchar *main_dirname = g_build_filename (CONF_DATADIR, DATABASE_SUBDIR, NULL);
+    const gchar *system_updates_dirname = g_build_filename ("/var/lib/lensfun-updates", DATABASE_SUBDIR, NULL);
 #else
+    /* windows based OS */
     extern gchar *_lf_get_database_dir ();
-    dirs [ndirs++] = _lf_get_database_dir ();
+    gchar *main_dirname = _lf_get_database_dir ();
+    const gchar *system_updates_dirname = "C:\\to\\be\\defined\\lensfun-updates";
 #endif
-    int static_ndirs = ndirs;
-    for (tmp = g_get_system_data_dirs (); ndirs < 10 && *tmp; tmp++)
-        dirs [ndirs++] = g_build_filename (*tmp, CONF_PACKAGE, NULL);
+    const int timestamp_main =
+        _lf_read_database_timestamp (main_dirname);
+    const int timestamp_system_updates =
+        _lf_read_database_timestamp (system_updates_dirname);
+    const int timestamp_user_updates =
+        _lf_read_database_timestamp (UserUpdatesDir);
+    if (timestamp_main > timestamp_system_updates)
+        if (timestamp_user_updates > timestamp_main)
+            database_found |= LoadDirectory (UserUpdatesDir);
+        else
+            database_found |= LoadDirectory (main_dirname);
+    else
+        if (timestamp_user_updates > timestamp_system_updates)
+            database_found |= LoadDirectory (UserUpdatesDir);
+        else
+            database_found |= LoadDirectory (system_updates_dirname);
+    g_free (main_dirname);
 
-    while (ndirs > 0)
-    {
-        ndirs--;
-        GDir *dir = g_dir_open (dirs [ndirs], 0, NULL);
+    database_found |= LoadDirectory (HomeDataDir);
 
-        if (dir)
-        {
-            GPatternSpec *ps = g_pattern_spec_new ("*.xml");
-            if (ps)
-            {
-                const gchar *fn;
-                while ((fn = g_dir_read_name (dir)))
-                {
-                    size_t sl = strlen (fn);
-                    if (g_pattern_match (ps, sl, fn, NULL))
-                    {
-                        gchar *ffn = g_build_filename (dirs [ndirs], fn, NULL);
-                        /* Ignore errors */
-                        Load (ffn);
-                        g_free (ffn);
-                    }
-                }
-                g_pattern_spec_free (ps);
-            }
-            g_dir_close (dir);
-        }
-
-        /* Free all paths except the first one which is held in db struct */
-        if (ndirs >= static_ndirs)
-            g_free (dirs [ndirs]);
-    }
-
-    return LF_NO_ERROR;
+    return database_found ? LF_NO_ERROR : LF_NO_DATABASE;
 }
 
 lfError lfDatabase::Load (const char *filename)
@@ -100,15 +147,28 @@ lfError lfDatabase::Load (const char *filename)
 /* Private structure used by XML parse */
 typedef struct
 {
-    lfExtDatabase *db;
+    lfDatabase *db;
     lfMount *mount;
     lfCamera *camera;
     lfLens *lens;
     gchar *lang;
     const gchar *stack [16];
     size_t stack_depth;
+    const char *errcontext;
 } lfParserData;
 
+static bool __chk_no_attrs(const gchar *element_name, const gchar **attribute_names,
+                           GError **error)
+{
+    if (attribute_names [0])
+    {
+        g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
+                     "The <%s> element cannot have any attributes!\n", element_name);
+        return false;
+    }
+    return true;
+}
+
 static void _xml_start_element (GMarkupParseContext *context,
                                 const gchar         *element_name,
                                 const gchar        **attribute_names,
@@ -141,46 +201,58 @@ static void _xml_start_element (GMarkupParseContext *context,
             return;
         }
 
-    chk_no_attrs:
-        if (attribute_names [0])
+        int version = 0;
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "version"))
+                version = atoi (attribute_values [i]);
+            else
+                goto bad_attr;
+        if (version > LF_MAX_DATABASE_VERSION)
         {
-            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                         "The <%s> element cannot have any attributes!\n",
-                         element_name);
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                         "Database version is %d, but supported is only %d!\n",
+                         version, LF_MAX_DATABASE_VERSION);
             return;
         }
     }
     else if (!strcmp (element_name, "mount"))
     {
-        if (!strcmp (ctx, "lensdatabase"))
+        if (ctx && !strcmp (ctx, "lensdatabase"))
         {
             pd->mount = new lfMount ();
-            goto chk_no_attrs;
+            if (!__chk_no_attrs(element_name, attribute_names, error)) return;
+        }
+        else if (ctx &&
+                 (!strcmp (ctx, "camera") ||
+                  !strcmp (ctx, "lens")))
+        {
+            if (!__chk_no_attrs(element_name, attribute_names, error)) return;
         }
-        else if (!strcmp (ctx, "camera") ||
-                 !strcmp (ctx, "lens"))
-            goto chk_no_attrs;
         else
             goto bad_ctx;
     }
     else if (!strcmp (element_name, "camera"))
     {
-        if (strcmp (ctx, "lensdatabase"))
+        if (!ctx || strcmp (ctx, "lensdatabase"))
             goto bad_ctx;
         pd->camera = new lfCamera ();
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "lens"))
     {
-        if (strcmp (ctx, "lensdatabase"))
+        if (!ctx || strcmp (ctx, "lensdatabase"))
             goto bad_ctx;
         pd->lens = new lfLens ();
+        // Set default values for lens database entries.  This way, they are
+        // still 0 (aka "unknown") for dummy lfLens instances created ad hoc
+        // for search matching.
         pd->lens->Type = LF_RECTILINEAR;
-        goto chk_no_attrs;
+        pd->lens->AspectRatio = 1.5;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "focal"))
     {
-        if (strcmp (ctx, "lens") || !pd->lens)
+        if (!ctx || strcmp (ctx, "lens") || !pd->lens)
             goto bad_ctx;
 
         for (i = 0; attribute_names [i]; i++)
@@ -195,7 +267,7 @@ static void _xml_start_element (GMarkupParseContext *context,
     }
     else if (!strcmp (element_name, "aperture"))
     {
-        if (strcmp (ctx, "lens") || !pd->lens)
+        if (!ctx || strcmp (ctx, "lens") || !pd->lens)
             goto bad_ctx;
 
         for (i = 0; attribute_names [i]; i++)
@@ -210,7 +282,7 @@ static void _xml_start_element (GMarkupParseContext *context,
     }
     else if (!strcmp (element_name, "center"))
     {
-        if (strcmp (ctx, "lens") || !pd->lens)
+        if (!ctx || strcmp (ctx, "lens") || !pd->lens)
             goto bad_ctx;
 
         for (i = 0; attribute_names [i]; i++)
@@ -221,36 +293,21 @@ static void _xml_start_element (GMarkupParseContext *context,
             else
                 goto bad_attr;
     }
-    else if (!strcmp (element_name, "cci"))
-    {
-        if (strcmp (ctx, "lens") || !pd->lens)
-            goto bad_ctx;
-
-        for (i = 0; attribute_names [i]; i++)
-            if (!strcmp (attribute_names [i], "red"))
-                pd->lens->RedCCI = atof (attribute_values [i]);
-            else if (!strcmp (attribute_names [i], "green"))
-                pd->lens->GreenCCI = atof (attribute_values [i]);
-            else if (!strcmp (attribute_names [i], "blue"))
-                pd->lens->BlueCCI = atof (attribute_values [i]);
-            else
-                goto bad_attr;
-    }
     else if (!strcmp (element_name, "type"))
     {
-        if (strcmp (ctx, "lens") || !pd->lens)
+        if (!ctx || strcmp (ctx, "lens") || !pd->lens)
             goto bad_ctx;
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "calibration"))
     {
-        if (strcmp (ctx, "lens"))
+        if (!ctx || strcmp (ctx, "lens"))
             goto bad_ctx;
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "distortion"))
     {
-        if (strcmp (ctx, "calibration"))
+        if (!ctx || strcmp (ctx, "calibration"))
             goto bad_ctx;
 
         lfLensCalibDistortion dc;
@@ -264,8 +321,6 @@ static void _xml_start_element (GMarkupParseContext *context,
                     dc.Model = LF_DIST_MODEL_POLY3;
                 else if (!strcmp (attribute_values [i], "poly5"))
                     dc.Model = LF_DIST_MODEL_POLY5;
-                else if (!strcmp (attribute_values [i], "fov1"))
-                    dc.Model = LF_DIST_MODEL_FOV1;
                 else if (!strcmp (attribute_values [i], "ptlens"))
                     dc.Model = LF_DIST_MODEL_PTLENS;
                 else
@@ -280,8 +335,7 @@ static void _xml_start_element (GMarkupParseContext *context,
             else if (!strcmp (attribute_names [i], "focal"))
                 dc.Focal = atof (attribute_values [i]);
             else if (!strcmp (attribute_names [i], "a") ||
-                     !strcmp (attribute_names [i], "k1") ||
-                     !strcmp (attribute_names [i], "omega"))
+                     !strcmp (attribute_names [i], "k1"))
                 dc.Terms [0] = atof (attribute_values [i]);
             else if (!strcmp (attribute_names [i], "b") ||
                      !strcmp (attribute_names [i], "k2"))
@@ -301,7 +355,7 @@ static void _xml_start_element (GMarkupParseContext *context,
     }
     else if (!strcmp (element_name, "tca"))
     {
-        if (strcmp (ctx, "calibration"))
+        if (!ctx || strcmp (ctx, "calibration"))
             goto bad_ctx;
 
         lfLensCalibTCA tcac;
@@ -342,7 +396,7 @@ static void _xml_start_element (GMarkupParseContext *context,
     }
     else if (!strcmp (element_name, "vignetting"))
     {
-        if (strcmp (ctx, "calibration"))
+        if (!ctx || strcmp (ctx, "calibration"))
             goto bad_ctx;
 
         lfLensCalibVignetting vc;
@@ -374,6 +428,87 @@ static void _xml_start_element (GMarkupParseContext *context,
 
         pd->lens->AddCalibVignetting (&vc);
     }
+    else if (!strcmp (element_name, "crop"))
+    {
+        if (!ctx || strcmp (ctx, "calibration"))
+            goto bad_ctx;
+
+        lfLensCalibCrop lcc;
+        memset (&lcc, 0, sizeof (lcc));
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "focal"))
+                lcc.Focal = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "mode"))
+            {
+                if (!strcmp (attribute_values [i], "no_crop"))
+                    lcc.CropMode = LF_NO_CROP;
+                else if (!strcmp (attribute_values [i], "crop_rectangle"))
+                    lcc.CropMode = LF_CROP_RECTANGLE;
+                else if (!strcmp (attribute_values [i], "crop_circle"))
+                    lcc.CropMode = LF_CROP_CIRCLE;
+                else
+                {
+                    goto bad_attr;
+                }
+            }
+            else if (!strcmp (attribute_names [i], "left"))
+                lcc.Crop [0] = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "right"))
+                lcc.Crop [1] = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "top"))
+                lcc.Crop [2] = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "bottom"))
+                lcc.Crop [3] = atof (attribute_values [i]);
+            else
+            {
+                goto unk_attr;
+            }
+
+        pd->lens->AddCalibCrop (&lcc);
+    }
+    else if (!strcmp (element_name, "field_of_view"))
+    {
+        if (!ctx || strcmp (ctx, "calibration"))
+            goto bad_ctx;
+
+        gint line, col;
+        g_markup_parse_context_get_position (context, &line, &col);
+        g_warning ("[Lensfun] %s:%d:%d: <field_of_view> tag is deprecated.  Use <real-focal-length> instead",
+                   pd->errcontext, line, col);
+
+        lfLensCalibFov lcf;
+        memset (&lcf, 0, sizeof (lcf));
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "focal"))
+                lcf.Focal = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "fov"))
+                lcf.FieldOfView = atof (attribute_values [i]);
+            else
+            {
+                goto unk_attr;
+            }
+
+        pd->lens->AddCalibFov (&lcf);
+    }
+    else if (!strcmp (element_name, "real-focal-length"))
+    {
+        if (!ctx || strcmp (ctx, "calibration"))
+            goto bad_ctx;
+
+        lfLensCalibRealFocal lcf;
+        memset (&lcf, 0, sizeof (lcf));
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "focal"))
+                lcf.Focal = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "real-focal"))
+                lcf.RealFocal = atof (attribute_values [i]);
+            else
+            {
+                goto unk_attr;
+            }
+
+        pd->lens->AddCalibRealFocal (&lcf);
+    }
     /* Handle multi-language strings */
     else if (!strcmp (element_name, "maker") ||
              !strcmp (element_name, "model") ||
@@ -387,8 +522,11 @@ static void _xml_start_element (GMarkupParseContext *context,
                 goto unk_attr;
     }
     else if (!strcmp (element_name, "compat") ||
-             !strcmp (element_name, "cropfactor"))
-        goto chk_no_attrs;
+             !strcmp (element_name, "cropfactor") ||
+             !strcmp (element_name, "aspect-ratio"))
+    {
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
+    }
     else
         g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
                      "Unknown element <%s>!\n", element_name);
@@ -419,8 +557,7 @@ static void _xml_end_element (GMarkupParseContext *context,
             return;
         }
 
-        _lf_ptr_array_insert_unique (
-            pd->db->Mounts, pd->mount, _lf_mount_compare, (GDestroyNotify)lf_mount_destroy);
+        pd->db->AddMount(pd->mount);
         pd->mount = NULL;
     }
     else if (!strcmp (element_name, "camera"))
@@ -435,8 +572,7 @@ static void _xml_end_element (GMarkupParseContext *context,
             return;
         }
 
-        _lf_ptr_array_insert_unique (
-            pd->db->Cameras, pd->camera, _lf_camera_compare, (GDestroyNotify)lf_camera_destroy);
+        pd->db->AddCamera(pd->camera);
         pd->camera = NULL;
     }
     else if (!strcmp (element_name, "lens"))
@@ -451,8 +587,7 @@ static void _xml_end_element (GMarkupParseContext *context,
             return;
         }
 
-        _lf_ptr_array_insert_unique (
-            pd->db->Lenses, pd->lens, _lf_lens_compare, (GDestroyNotify)lf_lens_destroy);
+        pd->db->AddLens(pd->lens);
         pd->lens = NULL;
     }
 }
@@ -534,6 +669,19 @@ static void _xml_text (GMarkupParseContext *context,
         else
             goto bad_ctx;
     }
+    else if (!strcmp (ctx, "aspect-ratio"))
+    {
+        if (pd->lens)
+        {
+            const char *colon = strpbrk (text, ":");
+            if (colon)
+                pd->lens->AspectRatio = atof (text) / atof (colon + 1);
+            else
+                pd->lens->AspectRatio = atof (text);
+        }
+        else
+            goto bad_ctx;
+    }
     else if (!strcmp (ctx, "type"))
     {
         if (pd->lens)
@@ -546,6 +694,14 @@ static void _xml_text (GMarkupParseContext *context,
                 pd->lens->Type = LF_PANORAMIC;
             else if (!_lf_strcmp (text, "equirectangular"))
                 pd->lens->Type = LF_EQUIRECTANGULAR;
+            else if (!_lf_strcmp (text, "orthographic"))
+                pd->lens->Type = LF_FISHEYE_ORTHOGRAPHIC;
+            else if (!_lf_strcmp (text, "stereographic"))
+                pd->lens->Type = LF_FISHEYE_STEREOGRAPHIC;
+            else if (!_lf_strcmp (text, "equisolid"))
+                pd->lens->Type = LF_FISHEYE_EQUISOLID;
+            else if (!_lf_strcmp (text, "fisheye_thoby"))
+                pd->lens->Type = LF_FISHEYE_THOBY;
             else
             {
                 g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -575,21 +731,22 @@ lfError lfDatabase::Load (const char *errcontext, const char *data, size_t data_
         NULL
     };
 
-    lfExtDatabase *This = static_cast<lfExtDatabase *> (this);
-
     /* Temporarily drop numeric format to "C" */
-    char *old_numeric = setlocale (LC_NUMERIC, "C");
+    char *old_numeric = setlocale (LC_NUMERIC, NULL);
+    old_numeric = strdup(old_numeric);
+    setlocale(LC_NUMERIC,"C");
 
     /* eek! GPtrArray does not have a method to insert a pointer
      into middle of the array... We have to remove the trailing
      NULL and re-append it after loading ... */
-    g_ptr_array_remove_index_fast (This->Mounts, This->Mounts->len - 1);
-    g_ptr_array_remove_index_fast (This->Cameras, This->Cameras->len - 1);
-    g_ptr_array_remove_index_fast (This->Lenses, This->Lenses->len - 1);
+    g_ptr_array_remove_index_fast ((GPtrArray *)Mounts, ((GPtrArray *)Mounts)->len - 1);
+    g_ptr_array_remove_index_fast ((GPtrArray *)Cameras, ((GPtrArray *)Cameras)->len - 1);
+    g_ptr_array_remove_index_fast ((GPtrArray *)Lenses, ((GPtrArray *)Lenses)->len - 1);
 
     lfParserData pd;
     memset (&pd, 0, sizeof (pd));
-    pd.db = This;
+    pd.db = this;
+    pd.errcontext = errcontext;
 
     GMarkupParseContext *mpc = g_markup_parse_context_new (
         &gmp, (GMarkupParseFlags)0, &pd, NULL);
@@ -603,29 +760,29 @@ lfError lfDatabase::Load (const char *errcontext, const char *data, size_t data_
     {
         gint line, col;
         g_markup_parse_context_get_position (mpc, &line, &col);
-        g_warning ("%s:%d:%d: %s", errcontext, line, col, err->message);
+        g_warning ("[Lensfun] %s:%d:%d: %s", errcontext, line, col, err->message);
     }
 
     g_markup_parse_context_free (mpc);
 
     /* Re-add the trailing NULL */
-    g_ptr_array_add (This->Mounts, NULL);
-    g_ptr_array_add (This->Cameras, NULL);
-    g_ptr_array_add (This->Lenses, NULL);
+    g_ptr_array_add ((GPtrArray *)Mounts, NULL);
+    g_ptr_array_add ((GPtrArray *)Cameras, NULL);
+    g_ptr_array_add ((GPtrArray *)Lenses, NULL);
 
     /* Restore numeric format */
     setlocale (LC_NUMERIC, old_numeric);
+    free(old_numeric);
 
     return e;
 }
 
 lfError lfDatabase::Save (const char *filename) const
 {
-    const lfExtDatabase *This = static_cast<const lfExtDatabase *> (this);
     return Save (filename,
-        (lfMount **)This->Mounts->pdata,
-        (lfCamera **)This->Cameras->pdata,
-        (lfLens **)This->Lenses->pdata);
+                 (lfMount **)((GPtrArray *)Mounts)->pdata,
+                 (lfCamera **)((GPtrArray *)Cameras)->pdata,
+                 (lfLens **)((GPtrArray *)Lenses)->pdata);
 }
 
 lfError lfDatabase::Save (const char *filename,
@@ -664,6 +821,11 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                         const lfCamera *const *cameras,
                         const lfLens *const *lenses)
 {
+    /* Temporarily drop numeric format to "C" */
+    char *old_numeric = setlocale (LC_NUMERIC, NULL);
+    old_numeric = strdup(old_numeric);
+    setlocale(LC_NUMERIC,"C");
+
     int i, j;
     GString *output = g_string_sized_new (1024);
 
@@ -734,25 +896,24 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                                 lenses [i]->Type == LF_FISHEYE ? "fisheye" :
                                 lenses [i]->Type == LF_PANORAMIC ? "panoramic" :
                                 lenses [i]->Type == LF_EQUIRECTANGULAR ? "equirectangular" :
+                                lenses [i]->Type == LF_FISHEYE_ORTHOGRAPHIC ? "orthographic" :
+                                lenses [i]->Type == LF_FISHEYE_STEREOGRAPHIC ? "stereographic" :
+                                lenses [i]->Type == LF_FISHEYE_EQUISOLID ? "equisolid" :
+                                lenses [i]->Type == LF_FISHEYE_THOBY ? "fisheye_thoby" :
                                 "rectilinear");
 
             if (lenses [i]->CenterX || lenses [i]->CenterY)
                 _lf_xml_printf (output, "\t\t<center x=\"%g\" y=\"%g\" />\n",
                                 lenses [i]->CenterX, lenses [i]->CenterY);
 
-            if (lenses [i]->RedCCI != 0 ||
-                lenses [i]->GreenCCI != 5 ||
-                lenses [i]->BlueCCI != 4)
-                _lf_xml_printf (output, "\t\t<cci red=\"%g\" green=\"%g\" blue=\"%g\" />\n",
-                                lenses [i]->RedCCI,
-                                lenses [i]->GreenCCI,
-                                lenses [i]->BlueCCI);
-
             _lf_xml_printf (output, "\t\t<cropfactor>%g</cropfactor>\n",
                             lenses [i]->CropFactor);
+            _lf_xml_printf (output, "\t\t<aspect-ratio>%g</aspect-ratio>\n",
+                            lenses [i]->AspectRatio);
 
             if (lenses [i]->CalibDistortion || lenses [i]->CalibTCA ||
-                lenses [i]->CalibVignetting)
+                lenses [i]->CalibVignetting || lenses [i]->CalibCrop || 
+                lenses [i]->CalibFov || lenses [i]->CalibRealFocal)
                 g_string_append (output, "\t\t<calibration>\n");
 
             if (lenses [i]->CalibDistortion)
@@ -777,12 +938,6 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                                 cd->Terms [0], cd->Terms [1]);
                             break;
 
-                        case LF_DIST_MODEL_FOV1:
-                            _lf_xml_printf (
-                                output, "model=\"fov1\" omega=\"%g\" />\n",
-                                cd->Terms [0]);
-                            break;
-
                         case LF_DIST_MODEL_PTLENS:
                             _lf_xml_printf (
                                 output, "model=\"ptlens\" a=\"%g\" b=\"%g\" c=\"%g\" />\n",
@@ -809,6 +964,13 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                                             ctca->Terms [0], ctca->Terms [1]);
                             break;
 
+                        case LF_TCA_MODEL_POLY3:
+                            _lf_xml_printf (output, "model=\"poly3\" vr=\"%g\" vb=\"%g\" "
+                                            "cr=\"%g\" cb=\"%g\" br=\"%g\" bb=\"%g\" />\n",
+                                            ctca->Terms [0], ctca->Terms [1], ctca->Terms [2],
+                                            ctca->Terms [3], ctca->Terms [4], ctca->Terms [5]);
+                            break;
+
                         default:
                             _lf_xml_printf (output, "model=\"none\" />\n");
                             break;
@@ -837,8 +999,67 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                 }
             }
 
+            if (lenses [i]->CalibCrop)
+            {
+                for (j = 0; lenses [i]->CalibCrop [j]; j++)
+                {
+                    lfLensCalibCrop *lcc = lenses [i]->CalibCrop [j];
+
+                    _lf_xml_printf (output, "\t\t\t<crop focal=\"%g\" ",
+                                    lcc->Focal);
+                    switch (lcc->CropMode)
+                    {
+                        case LF_CROP_RECTANGLE:
+                            _lf_xml_printf (
+                                output, "mode=\"crop_rectangle\" left=\"%g\" right=\"%g\" top=\"%g\" bottom=\"%g\" />\n",
+                                lcc->Crop [0], lcc->Crop [1], lcc->Crop [2], lcc->Crop [3]);
+                            break;
+
+                        case LF_CROP_CIRCLE:
+                            _lf_xml_printf (
+                                output, "mode=\"crop_circle\" left=\"%g\" right=\"%g\" top=\"%g\" bottom=\"%g\" />\n",
+                                lcc->Crop [0], lcc->Crop [1], lcc->Crop [2], lcc->Crop [3]);
+                            break;
+
+                        case LF_NO_CROP:
+                        default:
+                            _lf_xml_printf (output, "mode=\"no_crop\" />\n");
+                            break;
+                    }
+                }
+            }
+
+            if (lenses [i]->CalibFov)
+            {
+                for (j = 0; lenses [i]->CalibFov [j]; j++)
+                {
+                    lfLensCalibFov *lcf = lenses [i]->CalibFov [j];
+
+                    if (lcf->FieldOfView>0)
+                    {
+                        _lf_xml_printf (output, "\t\t\t<field_of_view focal=\"%g\" fov=\"%g\" />\n",
+                            lcf->Focal, lcf->FieldOfView);
+                    };
+                }
+            }
+
+            if (lenses [i]->CalibRealFocal)
+            {
+                for (j = 0; lenses [i]->CalibRealFocal [j]; j++)
+                {
+                    lfLensCalibRealFocal *lcf = lenses [i]->CalibRealFocal [j];
+
+                    if (lcf->RealFocal>0)
+                    {
+                        _lf_xml_printf (output, "\t\t\t<real-focal-length focal=\"%g\" real-focal=\"%g\" />\n",
+                            lcf->Focal, lcf->RealFocal);
+                    };
+                }
+            }
+
             if (lenses [i]->CalibDistortion || lenses [i]->CalibTCA ||
-                lenses [i]->CalibVignetting)
+                lenses [i]->CalibVignetting || lenses [i]->CalibCrop ||
+                lenses [i]->CalibFov || lenses [i]->CalibRealFocal)
                 g_string_append (output, "\t\t</calibration>\n");
 
             g_string_append (output, "\t</lens>\n\n");
@@ -846,6 +1067,10 @@ char *lfDatabase::Save (const lfMount *const *mounts,
 
     g_string_append (output, "</lensdatabase>\n");
 
+    /* Restore numeric format */
+    setlocale (LC_NUMERIC, old_numeric);
+    free(old_numeric);
+
     return g_string_free (output, FALSE);
 }
 
@@ -874,27 +1099,26 @@ const lfCamera **lfDatabase::FindCameras (const char *maker, const char *model)
     if (model && !*model)
         model = NULL;
 
-    const GPtrArray *cameras = static_cast<const lfExtDatabase *> (this)->Cameras;
     lfCamera tc;
     tc.SetMaker (maker);
     tc.SetModel (model);
-    int idx = _lf_ptr_array_find_sorted (cameras, &tc, __find_camera_compare);
+    int idx = _lf_ptr_array_find_sorted ((GPtrArray *)Cameras, &tc, __find_camera_compare);
     if (idx < 0)
         return NULL;
 
     guint idx1 = idx;
     while (idx1 > 0 &&
-           __find_camera_compare (g_ptr_array_index (cameras, idx1 - 1), &tc) == 0)
+           __find_camera_compare (g_ptr_array_index ((GPtrArray *)Cameras, idx1 - 1), &tc) == 0)
         idx1--;
 
     guint idx2 = idx;
-    while (++idx2 < cameras->len - 1 &&
-           __find_camera_compare (g_ptr_array_index (cameras, idx2), &tc) == 0)
+    while (++idx2 < ((GPtrArray *)Cameras)->len - 1 &&
+           __find_camera_compare (g_ptr_array_index ((GPtrArray *)Cameras, idx2), &tc) == 0)
         ;
 
     const lfCamera **ret = g_new (const lfCamera *, idx2 - idx1 + 1);
     for (guint i = idx1; i < idx2; i++)
-        ret [i - idx1] = (lfCamera *)g_ptr_array_index (cameras, i);
+        ret [i - idx1] = (lfCamera *)g_ptr_array_index ((GPtrArray *)Cameras, i);
     ret [idx2 - idx1] = NULL;
     return ret;
 }
@@ -915,15 +1139,14 @@ const lfCamera **lfDatabase::FindCamerasExt (const char *maker, const char *mode
     if (model && !*model)
         model = NULL;
 
-    const GPtrArray *cameras = static_cast<const lfExtDatabase *> (this)->Cameras;
     GPtrArray *ret = g_ptr_array_new ();
 
     lfFuzzyStrCmp fcmaker (maker, (sflags & LF_SEARCH_LOOSE) == 0);
     lfFuzzyStrCmp fcmodel (model, (sflags & LF_SEARCH_LOOSE) == 0);
 
-    for (size_t i = 0; i < cameras->len - 1; i++)
+    for (size_t i = 0; i < ((GPtrArray *)Cameras)->len - 1; i++)
     {
-        lfCamera *dbcam = static_cast<lfCamera *> (g_ptr_array_index (cameras, i));
+        lfCamera *dbcam = static_cast<lfCamera *> (g_ptr_array_index ((GPtrArray *)Cameras, i));
         int score1 = 0, score2 = 0;
         if ((!maker || (score1 = fcmaker.Compare (dbcam->Maker))) &&
             (!model || (score2 = fcmodel.Compare (dbcam->Model))))
@@ -943,8 +1166,7 @@ const lfCamera **lfDatabase::FindCamerasExt (const char *maker, const char *mode
 
 const lfCamera *const *lfDatabase::GetCameras () const
 {
-    const lfExtDatabase *This = static_cast<const lfExtDatabase *> (this);
-    return (lfCamera **)This->Cameras->pdata;
+    return (lfCamera **)((GPtrArray *)Cameras)->pdata;
 }
 
 const lfLens **lfDatabase::FindLenses (const lfCamera *camera,
@@ -975,6 +1197,20 @@ static gint _lf_compare_lens_score (gconstpointer a, gconstpointer b)
     return i2->Score - i1->Score;
 }
 
+static gint _lf_compare_lens_details (gconstpointer a, gconstpointer b)
+{
+    // Actually, we not only sort by focal length, but by MinFocal, MaxFocal,
+    // MinAperature, Maker, and Model -- in this order of priorities.
+    lfLens *i1 = (lfLens *)a;
+    lfLens *i2 = (lfLens *)b;
+
+    int cmp = _lf_lens_parameters_compare (i1, i2);
+    if (cmp != 0)
+        return cmp;
+
+    return _lf_lens_name_compare (i1, i2);
+}
+
 static void _lf_add_compat_mounts (
     const lfDatabase *This, const lfLens *lens, GPtrArray *mounts, char *mount)
 {
@@ -1003,7 +1239,6 @@ static void _lf_add_compat_mounts (
 
 const lfLens **lfDatabase::FindLenses (const lfLens *lens, int sflags) const
 {
-    const GPtrArray *lenses = static_cast<const lfExtDatabase *> (this)->Lenses;
     GPtrArray *ret = g_ptr_array_new ();
     GPtrArray *mounts = g_ptr_array_new ();
 
@@ -1016,14 +1251,32 @@ const lfLens **lfDatabase::FindLenses (const lfLens *lens, int sflags) const
     g_ptr_array_add (mounts, NULL);
 
     int score;
-    for (size_t i = 0; i < lenses->len - 1; i++)
+    const bool sort_and_uniquify = (sflags & LF_SEARCH_SORT_AND_UNIQUIFY) != 0;
+    for (size_t i = 0; i < ((GPtrArray *)Lenses)->len - 1; i++)
     {
-        lfLens *dblens = static_cast<lfLens *> (g_ptr_array_index (lenses, i));
+        lfLens *dblens = static_cast<lfLens *> (g_ptr_array_index ((GPtrArray *)Lenses, i));
         if ((score = _lf_lens_compare_score (
             lens, dblens, &fc, (const char **)mounts->pdata)) > 0)
         {
             dblens->Score = score;
-            _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_score);
+            if (sort_and_uniquify) {
+                bool already = false;
+                for (size_t i = 0; i < ret->len; i++)
+                {
+                    const lfLens *previous_lens = static_cast<lfLens *> (g_ptr_array_index (ret, i));
+                    if (!_lf_lens_name_compare (previous_lens, dblens))
+                    {
+                        if (dblens->Score > previous_lens->Score)
+                            ret->pdata[i] = dblens;
+                        already = true;
+                        break;
+                    }
+                }
+                if (!already)
+                    _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_details);
+            }
+            else
+                _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_score);
         }
     }
 
@@ -1039,20 +1292,18 @@ const lfLens **lfDatabase::FindLenses (const lfLens *lens, int sflags) const
 
 const lfLens *const *lfDatabase::GetLenses () const
 {
-    const lfExtDatabase *This = static_cast<const lfExtDatabase *> (this);
-    return (lfLens **)This->Lenses->pdata;
+    return (lfLens **)((GPtrArray *)Lenses)->pdata;
 }
 
 const lfMount *lfDatabase::FindMount (const char *mount) const
 {
-    const GPtrArray *Mounts = static_cast<const lfExtDatabase *> (this)->Mounts;
     lfMount tm;
     tm.SetName (mount);
-    int idx = _lf_ptr_array_find_sorted (Mounts, &tm, _lf_mount_compare);
+    int idx = _lf_ptr_array_find_sorted ((GPtrArray *)Mounts, &tm, _lf_mount_compare);
     if (idx < 0)
         return NULL;
 
-    return (const lfMount *)g_ptr_array_index (Mounts, idx);
+    return (const lfMount *)g_ptr_array_index ((GPtrArray *)Mounts, idx);
 }
 
 const char *lfDatabase::MountName (const char *mount) const
@@ -1065,8 +1316,25 @@ const char *lfDatabase::MountName (const char *mount) const
 
 const lfMount * const *lfDatabase::GetMounts () const
 {
-    const lfExtDatabase *This = static_cast<const lfExtDatabase *> (this);
-    return (lfMount **)This->Mounts->pdata;
+    return (lfMount **)((GPtrArray *)Mounts)->pdata;
+}
+
+void lfDatabase::AddMount (lfMount *mount)
+{
+    _lf_ptr_array_insert_unique (
+        (GPtrArray *)Mounts, mount, _lf_mount_compare, (GDestroyNotify)lf_mount_destroy);
+}
+
+void lfDatabase::AddCamera (lfCamera *camera)
+{
+    _lf_ptr_array_insert_unique (
+        (GPtrArray *)Cameras, camera, _lf_camera_compare, (GDestroyNotify)lf_camera_destroy);
+}
+
+void lfDatabase::AddLens (lfLens *lens)
+{
+    _lf_ptr_array_insert_unique (
+        (GPtrArray *)Lenses, lens, _lf_lens_compare, (GDestroyNotify)lf_lens_destroy);
 }
 
 //---------------------------// The C interface //---------------------------//
