@@ -791,6 +791,35 @@ static int __insert_spline (void **spline, float *spline_dist, float dist, void
     return -1;
 }
 
+static void __parameter_scales (float values [], int number_of_values,
+                                int type, int model, int index)
+{
+    switch (type)
+    {
+    case LF_MODIFY_DISTORTION:
+        switch (model)
+        {
+        case LF_DIST_MODEL_POLY3:
+        case LF_DIST_MODEL_POLY5:
+        case LF_DIST_MODEL_PTLENS:
+            break;
+        }
+        break;
+
+    case LF_MODIFY_TCA:
+        switch (model)
+        {
+        case LF_TCA_MODEL_LINEAR:
+        case LF_TCA_MODEL_POLY3:
+            if (index < 2)
+                for (int i=0; i < number_of_values; i++)
+                    values [i] = 1.0;
+            break;
+        }
+        break;
+    }
+}
+
 bool lfLens::InterpolateDistortion (float focal, lfLensCalibDistortion &res) const
 {
     if (!CalibDistortion)
@@ -816,7 +845,7 @@ bool lfLens::InterpolateDistortion (float focal, lfLensCalibDistortion &res) con
             dm = c->Model;
         else if (dm != c->Model)
         {
-            g_warning ("[lensfun] lens %s/%s has multiple distortion models defined\n",
+            g_warning ("[Lensfun] lens %s/%s has multiple distortion models defined\n",
                        Maker, Model);
             continue;
         }
@@ -851,10 +880,17 @@ bool lfLens::InterpolateDistortion (float focal, lfLensCalibDistortion &res) con
     float t = (focal - spline [1]->Focal) / (spline [2]->Focal - spline [1]->Focal);
 
     for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
+    {
+        float values [5] = {spline [0] ? spline [0]->Focal : NAN, spline [1]->Focal,
+                            spline [2]->Focal, spline [3] ? spline [3]->Focal : NAN,
+                            focal};
+        __parameter_scales (values, 5, LF_MODIFY_DISTORTION, dm, i);
         res.Terms [i] = _lf_interpolate (
-            spline [0] ? spline [0]->Terms [i] : FLT_MAX,
-            spline [1]->Terms [i], spline [2]->Terms [i],
-            spline [3] ? spline [3]->Terms [i] : FLT_MAX, t);
+            spline [0] ? spline [0]->Terms [i] * values [0] : FLT_MAX,
+            spline [1]->Terms [i] * values [1], spline [2]->Terms [i] * values [2],
+            spline [3] ? spline [3]->Terms [i] * values [3] : FLT_MAX,
+            t) / values [4];
+    }
 
     return true;
 }
@@ -884,7 +920,7 @@ bool lfLens::InterpolateTCA (float focal, lfLensCalibTCA &res) const
             tcam = c->Model;
         else if (tcam != c->Model)
         {
-            g_warning ("[lensfun] lens %s/%s has multiple TCA models defined\n",
+            g_warning ("[Lensfun] lens %s/%s has multiple TCA models defined\n",
                        Maker, Model);
             continue;
         }
@@ -919,10 +955,17 @@ bool lfLens::InterpolateTCA (float focal, lfLensCalibTCA &res) const
     float t = (focal - spline [1]->Focal) / (spline [2]->Focal - spline [1]->Focal);
 
     for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
+    {
+        float values [5] = {spline [0] ? spline [0]->Focal : NAN, spline [1]->Focal,
+                            spline [2]->Focal, spline [3] ? spline [3]->Focal : NAN,
+                            focal};
+        __parameter_scales (values, 5, LF_MODIFY_TCA, tcam, i);
         res.Terms [i] = _lf_interpolate (
-            spline [0] ? spline [0]->Terms [i] : FLT_MAX,
-            spline [1]->Terms [i], spline [2]->Terms [i],
-            spline [3] ? spline [3]->Terms [i] : FLT_MAX, t);
+            spline [0] ? spline [0]->Terms [i] * values [0] : FLT_MAX,
+            spline [1]->Terms [i] * values [1], spline [2]->Terms [i] * values [2],
+            spline [3] ? spline [3]->Terms [i] * values [3] : FLT_MAX,
+            t) / values [4];
+    }
 
     return true;
 }
@@ -940,8 +983,8 @@ static float __vignetting_dist (
         f1 /= df;
         f2 /= df;
     }
-    float a1 = 8.0 / aperture;
-    float a2 = 8.0 / x.Aperture;
+    float a1 = 4.0 / aperture;
+    float a2 = 4.0 / x.Aperture;
     float d1 = 0.1 / distance;
     float d2 = 0.1 / x.Distance;
 
@@ -978,7 +1021,7 @@ bool lfLens::InterpolateVignetting (
         } 
         else if (vm != c->Model)
         {
-            g_warning ("[lensfun] lens %s/%s has multiple vignetting models defined\n",
+            g_warning ("[Lensfun] lens %s/%s has multiple vignetting models defined\n",
                        Maker, Model);
             continue;
         }
@@ -1033,7 +1076,7 @@ bool lfLens::InterpolateCrop (float focal, lfLensCalibCrop &res) const
             cm = c->CropMode;
         else if (cm != c->CropMode)
         {
-            g_warning ("[lensfun] lens %s/%s has multiple crop modes defined\n",
+            g_warning ("[Lensfun] lens %s/%s has multiple crop modes defined\n",
                        Maker, Model);
             continue;
         }
